@@ -45,9 +45,6 @@
 #define MONOSPACE_FONT_DIR "/desktop/gnome/interface"
 #define MONOSPACE_FONT_KEY MONOSPACE_FONT_DIR "/monospace_font_name"
 #define HTTP_PROXY_DIR "/system/http_proxy"
-#define HTTP_PROXY_HOST HTTP_PROXY_DIR "/host"
-#define HTTP_PROXY_PORT HTTP_PROXY_DIR "/port"
-#define HTTP_PROXY_USE_PROXY HTTP_PROXY_DIR "/use_http_proxy"
 
 struct _TerminalScreenPrivate
 {
@@ -56,7 +53,6 @@ struct _TerminalScreenPrivate
   TerminalProfile *profile; /* may be NULL at times */
   guint profile_changed_id;
   guint profile_forgotten_id;
-  int id;
   GtkWidget *popup_menu;
   char *raw_title, *raw_icon_title;
   char *cooked_title, *cooked_icon_title;
@@ -76,8 +72,6 @@ struct _TerminalScreenPrivate
   gboolean user_title; /* title was manually set */
 };
 
-static GList* used_ids = NULL;
-
 enum {
   PROFILE_SET,
   TITLE_CHANGED,
@@ -87,9 +81,8 @@ enum {
   LAST_SIGNAL
 };
 
-enum
-{
-  FLAVOR_AS_IS = 0,
+enum {
+  FLAVOR_AS_IS,
   FLAVOR_DEFAULT_TO_HTTP,
   FLAVOR_EMAIL
 };
@@ -280,30 +273,23 @@ terminal_screen_init (TerminalScreen *screen)
 #define PASSCHARS "-A-Za-z0-9,?;.:/!%$^*&~\"#'"
 #define HOSTCHARS "-A-Za-z0-9"
 #define PATHCHARS "-A-Za-z0-9_$.+!*(),;:@&=?/~#%"
-#define SCHEME    "(news|telnet|nntp|https?|ftps?|webcal)"
+#define SCHEME    "(news:|telnet:|nntp:|file:/|https?:|ftps?:|webcal:)"
 #define USER      "[" USERCHARS "]+(:["PASSCHARS "]+)?"
 #define URLPATH   "/[" PATHCHARS "]*[^]'.}>) \t\r\n,\\\"]"
 
   terminal_widget_match_add (screen->priv->term,
-			     "\\<(" SCHEME "://(" USER "@)?)[" HOSTCHARS ".]+"
+			     "\\<" SCHEME "//(" USER "@)?[" HOSTCHARS ".]+"
 			     "(:[0-9]+)?(" URLPATH ")?\\>", FLAVOR_AS_IS);
 
-  terminal_widget_match_add (screen->priv->term,
-			     "\\<(file:///(" URLPATH ")?", FLAVOR_AS_IS);
-			     
-
   terminal_widget_match_add (screen->priv->term,
 			     "\\<(www|ftp)[" HOSTCHARS "]*\\.[" HOSTCHARS ".]+"
 			     "(:[0-9]+)?(" URLPATH ")?\\>",
 			     FLAVOR_DEFAULT_TO_HTTP);
 
   terminal_widget_match_add (screen->priv->term,
-			     "\\<mailto:[a-z0-9][a-z0-9.-]*@[a-z0-9][a-z0-9-]*"
-			     "(\\.[a-z0-9][a-z0-9-]*)+\\>", FLAVOR_AS_IS);
-
-  terminal_widget_match_add (screen->priv->term,
-			     "\\<[a-z0-9][a-z0-9.-]*@[a-z0-9][a-z0-9-]*"
-			     "(\\.[a-z0-9][a-z0-9-]*)+\\>", FLAVOR_EMAIL);
+			     "\\<(mailto:)?[a-z0-9][a-z0-9.-]*@[a-z0-9]"
+			     "[a-z0-9-]*(\\.[a-z0-9][a-z0-9-]*)+\\>",
+			     FLAVOR_EMAIL);
 
   terminal_widget_match_add (screen->priv->term,
 			     "\\<news:[-A-Z\\^_a-z{|}~!\"#$%&'()*+,./0-9;:=?`]+"
@@ -450,8 +436,6 @@ terminal_screen_finalize (GObject *object)
 
   g_object_unref (conf);
 
-  used_ids = g_list_remove (used_ids, GINT_TO_POINTER (screen->priv->id));  
-
   if (screen->priv->title_editor)
     gtk_widget_destroy (screen->priv->title_editor);
   
@@ -473,30 +457,10 @@ terminal_screen_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static int
-next_unused_id (void)
-{
-  /* can't this be changed into a static unsigned long int? These ids are never used anyways...*/
-  int i = 0;
-
-  while (g_list_find (used_ids, GINT_TO_POINTER (i)))
-    ++i;
-
-  return i;
-}
-
 TerminalScreen*
 terminal_screen_new (void)
 {
-  TerminalScreen *screen;
-
-  screen = g_object_new (TERMINAL_TYPE_SCREEN, NULL);
-  
-  screen->priv->id = next_unused_id ();
-
-  used_ids = g_list_prepend (used_ids, GINT_TO_POINTER (screen->priv->id));
-  
-  return screen;
+  return g_object_new (TERMINAL_TYPE_SCREEN, NULL);
 }
 
 static void
@@ -512,12 +476,6 @@ terminal_screen_map (GtkWidget *widget)
   GTK_WIDGET_CLASS (parent_class)->map (widget);
 }
 
-int
-terminal_screen_get_id (TerminalScreen *screen)
-{
-  return screen->priv->id;
-}
-
 TerminalWindow*
 terminal_screen_get_window (TerminalScreen *screen)
 {
@@ -655,7 +613,6 @@ reread_profile (TerminalScreen *screen)
   
   terminal_widget_set_delete_binding (term,
                                       terminal_profile_get_delete_binding (profile));
-
 }
 
 /**
@@ -835,7 +792,7 @@ terminal_screen_update_on_realize (GtkWidget      *term,
                                        pango_font_description_get_size (desc));
 
       terminal_widget_set_pango_font (term, desc,
-      					terminal_profile_get_no_aa_without_render(profile));
+				      terminal_profile_get_no_aa_without_render (profile));
 
       pango_font_description_free (desc);
     }
@@ -1118,19 +1075,16 @@ static char**
 get_child_environment (GtkWidget      *term,
                        TerminalScreen *screen)
 {
-  /* code from gnome-terminal, sort of. */
-  TerminalProfile *profile;
-  char **p;
-  int i;
-  char **retval;
+  gchar **p, **retval;
+  gint i;
+  GConfClient *conf;
+  gboolean use_proxy;
 #define EXTRA_ENV_VARS 8
-  
-  profile = screen->priv->profile;
 
   /* count env vars that are set */
   for (p = environ; *p; p++)
     ;
-  
+
   i = p - environ;
   retval = g_new (char *, i + 1 + EXTRA_ENV_VARS);
 
@@ -1143,7 +1097,7 @@ get_child_environment (GtkWidget      *term,
           (strncmp (*p, "TERM=", 5) == 0)    ||
           (strncmp (*p, "GNOME_DESKTOP_ICON=", 19) == 0) ||
           (strncmp (*p, "COLORTERM=", 10) == 0) ||
-	  (strncmp ( *p, "DISPLAY=", 8) == 0))
+	  (strncmp (*p, "DISPLAY=", 8) == 0))
         {
           /* nothing: do not copy */
         }
@@ -1156,35 +1110,85 @@ get_child_environment (GtkWidget      *term,
 
   retval[i] = g_strdup ("COLORTERM="EXECUTABLE_NAME);
   ++i;
+
   retval[i] = g_strdup ("TERM=xterm"); /* FIXME configurable later? */
   ++i;
+
   retval[i] = g_strdup_printf ("WINDOWID=%ld",
                                GDK_WINDOW_XWINDOW (term->window));
   ++i;
+
   retval[i] = g_strdup_printf ("DISPLAY=%s", 
 			       gdk_display_get_name(gtk_widget_get_display(term)));
   ++i;
   
-  /* Setup HTTP proxy according to gconf */
-  GConfClient *conf = gconf_client_get_default ();
-  gchar *host = gconf_client_get_string (
-    conf, HTTP_PROXY_HOST, NULL);
-  gint port = gconf_client_get_int (
-    conf, HTTP_PROXY_PORT, NULL);
-  gboolean use_proxy = gconf_client_get_bool (
-    conf, HTTP_PROXY_USE_PROXY, NULL);
+  conf = gconf_client_get_default ();
+  use_proxy = gconf_client_get_bool (conf, HTTP_PROXY_DIR "/use_http_proxy", NULL);
+
+  if (use_proxy && !getenv ("http_proxy"))
+    {
+      gchar *host;
+      gint port;
+      GSList *ignore;
+
+      host = gconf_client_get_string (conf, HTTP_PROXY_DIR "/host", NULL);
+      port = gconf_client_get_int (conf, HTTP_PROXY_DIR "/port", NULL);
+      ignore = gconf_client_get_list (conf, HTTP_PROXY_DIR "/ignore_hosts",
+				      GCONF_VALUE_STRING, NULL);
+
+      g_object_unref (conf);
+
+      if (host && port)
+	{
+	  retval[i] = g_strdup_printf ("http_proxy=http://%s:%d/", host, port);
+	  g_free (host);
+	  ++i;
+	}
+
+      if (ignore)
+	{
+	  /* code distantly based on gconf's */
+	  gchar *buf = NULL;
+	  guint bufsize = 64;
+	  guint cur = 0;
+
+	  buf = g_malloc (bufsize + 3);
+
+	  while (ignore != NULL)
+	    {
+	      guint len = strlen (ignore->data);
+
+	      if ((cur + len + 2) >= bufsize) /* +2 for '\0' and comma */
+		{
+		  bufsize = MAX(bufsize * 2, bufsize + len + 4); 
+		  buf = g_realloc (buf, bufsize + 3);
+		}
+
+	      g_assert (cur < bufsize);
+
+	      strcpy (&buf[cur], ignore->data);
+	      cur += len;
+
+	      g_assert(cur < bufsize);
+
+	      buf[cur] = ',';
+	      ++cur;
+
+	      g_assert(cur < bufsize);
+
+	      ignore = g_slist_next (ignore);
+	    }
+
+	  buf[cur-1] = '\0'; /* overwrites last comma */
+
+	  retval[i] = g_strdup_printf ("no_proxy=%s", buf);
+	  g_free (buf);
+	  ++i;
+	}
+    }
 
-  if (host && port && !getenv ("http_proxy") && use_proxy)
-    retval[i] = g_strdup_printf ("http_proxy=%s:%d", host, port);
-  else
-    retval[i] = g_strdup_printf ("");
-  
-  if (host)
-    g_free (host);
-  ++i;
-  
   retval[i] = NULL;
-  
+
   return retval;
 }
 
@@ -1370,7 +1374,10 @@ open_url (TerminalScreen *screen,
       url = g_strdup_printf ("http:%s", orig_url);
       break;
     case FLAVOR_EMAIL:
-      url = g_strdup_printf ("mailto:%s", orig_url);
+      if (strncmp ("mailto:", orig_url, 7))
+	url = g_strdup_printf ("mailto:%s", orig_url);
+      else
+	url = g_strdup (orig_url);
       break;
     case FLAVOR_AS_IS:
       url = g_strdup (orig_url);
@@ -1558,19 +1565,15 @@ popup_clipboard_request_callback (GtkClipboard *clipboard,
   screen = info->screen;
 
   if (!GTK_WIDGET_REALIZED (screen->priv->term)) 
-    {
-      goto cleanup;
-    }
+    goto cleanup;
 
   if (screen->priv->popup_menu)
-    {
-      gtk_widget_destroy (screen->priv->popup_menu);
-    }
+    gtk_widget_destroy (screen->priv->popup_menu);
 
   screen->priv->popup_menu = gtk_menu_new ();
   gtk_menu_set_accel_group (GTK_MENU (screen->priv->popup_menu),
                             terminal_accels_get_group_for_widget (screen->priv->popup_menu));
-  
+
   gtk_menu_attach_to_widget (GTK_MENU (screen->priv->popup_menu),
                              GTK_WIDGET (screen->priv->term),
                              popup_menu_detach);
@@ -1579,39 +1582,42 @@ popup_clipboard_request_callback (GtkClipboard *clipboard,
     {
       if (screen->priv->matched_flavor == FLAVOR_EMAIL)
 	{
-	  menu_item = append_menuitem (screen->priv->popup_menu,
-				       _("_Send Mail To..."),
-				       G_CALLBACK (open_url_callback),
-				       screen);
+	  if (strncmp ("mailto:", screen->priv->matched_string, 7))
+	    {
+	      menu_item = append_menuitem (screen->priv->popup_menu,
+					   _("_Send Mail To..."),
+					   G_CALLBACK (open_url_callback),
+					   screen);
 
-	  menu_item = append_menuitem (screen->priv->popup_menu,
-				       _("_Copy E-mail Address"),
-				       G_CALLBACK (copy_url_callback),
-				       screen);
-  	}
-      else
-	{
-	  menu_item = append_menuitem (screen->priv->popup_menu,
-				       _("_Open Link"),
-				       G_CALLBACK (open_url_callback),
-				       screen);
+	      menu_item = append_menuitem (screen->priv->popup_menu,
+					   _("_Copy E-mail Address"),
+					   G_CALLBACK (copy_url_callback),
+					   screen);
+	    }
+	  else
+	    {
+	      menu_item = append_menuitem (screen->priv->popup_menu,
+					   _("_Open Link"),
+					   G_CALLBACK (open_url_callback),
+					   screen);
 
-	  menu_item = append_menuitem (screen->priv->popup_menu,
-				       _("_Copy Link Address"),
-				       G_CALLBACK (copy_url_callback),
-				       screen);
+	      menu_item = append_menuitem (screen->priv->popup_menu,
+					   _("_Copy Link Address"),
+					   G_CALLBACK (copy_url_callback),
+					   screen);
+	    }
 	}
 
       menu_item = gtk_separator_menu_item_new ();
       gtk_widget_show (menu_item);
       gtk_menu_shell_append (GTK_MENU_SHELL (screen->priv->popup_menu), menu_item);
     }
- 
+
   menu_item = append_menuitem (screen->priv->popup_menu,
                                _("Open _Terminal"),
                                G_CALLBACK (new_window_callback),
                                screen);
-  
+
   menu_item = append_menuitem (screen->priv->popup_menu,
                                _("Open Ta_b"),
                                G_CALLBACK (new_tab_callback),
@@ -2069,12 +2075,20 @@ title_entry_changed (GtkWidget      *entry,
                      TerminalScreen *screen)
 {
   char *text;
+  gboolean userset;
 
   text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
 
-  screen->priv->user_title = TRUE;  
-  terminal_screen_set_dynamic_title (screen, text, TRUE);
-  terminal_screen_set_dynamic_icon_title (screen, text, TRUE);
+  /* The user set the title to nothing, let's understand that as a
+     request to revert to dynamically setting the title again. */
+  if (G_UNLIKELY (*text == '\0'))
+    screen->priv->user_title = FALSE;
+  else
+    {
+      screen->priv->user_title = TRUE;
+      terminal_screen_set_dynamic_title (screen, text, TRUE);
+      terminal_screen_set_dynamic_icon_title (screen, text, TRUE);
+    }
 
   g_free (text);
 }
